function [z,p] = zptool(action);
% interactive pole-zero plot with zplane and freq response
% files required: 	zptool.m (this file)
%			zp2tf2.m (zp2tf.m with a modification to allow non-conjugate zp locations)
%			pzop.m performs ops on zeros and poles (add, delete, etc)
%			makestr.m constructs strings for display of zeros and poles

% global variable for transfer of data among multiple zptool calls
global mid bot b a z p k h f do tol lims MP y undo index undosize;
global xlim ylim hphase hreal htype hstable hpole hmidrange Nmid Nbot N;
global hreloc himloc hmagloc hanglocrad hanglocdeg hb ha hz hp zpformat hfixedmag;
global hfixedang horder hanglocnorm;
global ha2 hb2 hz2 hp2 hscale;

% initialize when zptool is run
if nargin == 0
	zptool('initialize');

% note zptool recursively calls itself with different input arguments to perform
% multiple functions
elseif strcmp(action,'initialize')
	tol = 10^(-10); % tolerance for real or imag check
	undosize = 1024; % size of undo matrix; max number of zeros or poles is 511 (each)
	zpformat = 0; % start with zp display mag angle instead of real imag
	undo = zeros(undosize,1); % initialize undo matrix
	index = 2; % location of next entry (column) in undo matrix
	HTOP = []; HMID = []; HBOT = []; % initialize handles for 3 sets of axes
	Nmid = 128; Nbot = 128; % initial number of points for the mid and lower axes
	mid = 1; % start mid axis with mag in dB
	bot = 4; % start bot axis with phase (unwrapped)
	z = []; p = []; k = 1; % initial matrices to null
	xlim = [-1.5 1.5]; % define initial limits to zplane graph
	ylim = xlim; % zplane is square

	% create zptool figure window
	figure('Position',[5 40 1000 700],'Name','zptool','NumberTitle','off');

% mid controls
	base = 320;
	labelStr='Mag (dB)|Mag (lin)|Phase wrap|Phase unwrap|Group|Impulse|Step';
	callbackStr='global mid;mid = get(gco,''value'');zptool(''refresh'');';
	uicontrol('Style','popupmenu','Position',[5 base+50 90 30], ...
		'String',labelStr,'Interruptible','yes',...
		'value',mid,'Callback',callbackStr);
	uicontrol('String','Hold On','Position',[5 base+20 60 20],...
		'callback','global HMID; axes(findobj(gcf,''tag'',''axmid'')); hold on;');
	uicontrol('String','Hold Off','Position',[5 base 60 20],...
		'callback','global HMID; axes(findobj(gcf,''tag'',''axmid'')); hold off;');
	uicontrol('style','text','position',[70 base+10 15 15],'string','N:',...
		'backgroundcolor','black','foregroundcolor','white');
	uicontrol('style','edit','position',[90 base+10 40 15],...
		'userdata',128,'backgroundcolor',[.4 .4 .4],...
		'string','128','foregroundcolor','white','tag','Nmid');
	uicontrol('String','New Window','Position',[5 base+80 80 20],...
		'Callback','global do mid; do = mid; figure; zptool(''newmid'');');

% refresh button
	uicontrol('string','Refresh','position',[5 240 80 20],...
		'callback','zptool(''refresh'')');

% bot controls
	base = 100;
	callbackStr='global bot;bot = get(gco,''value'');zptool(''refresh'');return;';
	uicontrol('Style','popupmenu','Position',[5 base+50 90 30], ...
		'String',labelStr,'Interruptible','yes',...
		'value',bot,'Callback',callbackStr);
	uicontrol('String','Hold On','Position',[5 base+20 60 20],...
		'callback','global HBOT; axes(findobj(gcf,''tag'',''axbot'')); hold on;');
	uicontrol('String','Hold Off','Position',[5 base 60 20],...
		'callback','global HBOT; axes(findobj(gcf,''tag'',''axbot'')); hold off;');
	uicontrol('style','text','position',[70 base+10 15 15],'string','N:',...
		'backgroundcolor','black','foregroundcolor','white');
	uicontrol('style','edit','position',[90 base+10 40 15],...
		'userdata',128,'backgroundcolor',[.4 .4 .4],...
		'string','128','foregroundcolor','white','tag','Nbot');
	uicontrol('String','New Window','Position',[5 base+80 80 20],...
		'Callback','global do bot; do = bot; figure; zptool(''newbot'');');

% clear, exit, undo, and redo
	base = 930;
	uicontrol('String','clear','Position',[base 40 40 20],...
		'Callback','global z p;z = []; p = []; zptool(''change'');')
	uicontrol('String','Exit','Position',[base 10 40 20],...
		'Callback','zptool(''exit'');')
	uicontrol('String','undo','Position',[base 70 40 20],...
		'Callback','zptool(''undo'');')
	uicontrol('string','redo','position',[base 100 40 20],...
		'callback','zptool(''redo'');');

% zp menus, invert, AP, Center At, Zout, Zin, Reflect All, Order, fix mag,
% fix angle, scale
	base = 545;
	labelStr='Add X|Add X*|Increase Order|Add X,X*|X at 0,0|X on Re|';
	labelStr = [labelStr 'Delete X|Delete X,X*|Reflect X|Reflect X,X*|Fix Mag X|'];
	labelStr = [labelStr 'Fix Mag X,X*|Fix Ang X|Fix Ang X,X*|Fix Mag Ang X|'];
	labelStr = [labelStr 'Fix Mag Ang X,X*'];
	callbackStr='global p HTOP;axes(findobj(gcf,''tag'',''axtop''));';
	callbackStr=[callbackStr 'p = pzop(p); zptool(''change'');return;'];
	hpole = uicontrol('Style','popupmenu','Position',[10 base+65 110 30], ...
		'String',labelStr,'Interruptible','yes',...
		'Callback',callbackStr);
	labelStr='Add 0|Add 0*|Increase Order|Add 0,0*|0 at 0,0|0 on Re|';
	labelStr = [labelStr 'Delete 0|Delete 0,0*|Reflect 0|Reflect 0,0*|Fix Mag 0|'];
	labelStr = [labelStr 'Fix Mag 0,0*|Fix Ang 0|Fix Ang 0,0*|Fix Mag Ang 0|'];
	labelStr = [labelStr 'Fix Mag Ang 0,0*'];
	callbackStr='global z HTOP;axes(findobj(gcf,''tag'',''axtop''));';
	callbackStr=[callbackStr 'z = pzop(z); zptool(''change'');return;'];
	hzero = uicontrol('Style','popupmenu','Position',[125 base+65 110 30], ...
		'String',labelStr,'Interruptible','yes',...
		'Callback',callbackStr);
	uicontrol('String','invert','Position',[10 base+125 75 20],...
		'Callback','[z,p] = zptool(''invert'');');
	uicontrol('String','AP','Position',[85 base+125 75 20],...
		'Callback','[z,p] = zptool(''AP'');');
	uicontrol('String','Center At','Position',[160 base+125 75 20],...
		'Callback','zptool(''center'');');
	uicontrol('String','Zoom In','Position',[10 base+105 75 20],...
		'Callback','zptool(''zoomin'');');
	uicontrol('String','Zoom Out','Position',[85 base+105 75 20],...
		'Callback','zptool(''zoomout'');');
	uicontrol('String','Reflect All','Position',[160 base+105 75 20],...
		'Callback','zptool(''reflectall'');');
	uicontrol('String','Filtdemo','Position',[10 base+85 113 20],...
		'Callback','zptool(''filtdemo'');');
	uicontrol('String','Update zptool','Position',[123 base+85 112 20],...
		'Callback','zptool(''filtdemoupdate'');');
	uicontrol('style','text','string','Order:',...
		'Position',[10 base+40 60 15],'backgroundcolor','black',...
		'foregroundcolor','white');
	horder = uicontrol('style','edit','position',[75 base+40 40 15],...
		'userdata',1.0,'backgroundcolor',[.4 .4 .4],...
		'string','1','foregroundcolor','white');
	uicontrol('style','text','string','Fix Mag:',...
		'Position',[10 base+20 60 15],'backgroundcolor','black',...
		'foregroundcolor','white');
	hfixedmag = uicontrol('style','edit','position',[75 base+20 40 15],...
		'userdata',1.0,'backgroundcolor',[.4 .4 .4],...
		'string','1.0','foregroundcolor','white');
	uicontrol('style','text','string','Fix Ang:',...
		'Position',[10 base 60 15],'backgroundcolor','black',...
		'foregroundcolor','white');
	hfixedang = uicontrol('style','edit','position',[75 base 40 15],...
		'userdata',.5,'backgroundcolor',[.4 .4 .4],...
		'string','.5','foregroundcolor','white');
	uicontrol('string','Scale by:','position',[10 base-30 70 20],'callback',...
		'zptool(''scale'');');
	hscale = uicontrol('style','edit','string','2','position',...
		[85 base-28 30 15],'backgroundcolor',[.4 .4 .4],'foregroundcolor',...
		'white','horizontalalignment','left');

% format, b a display, z p display
	basex = 600; basey = 440; height = 220;
	uicontrol('String','Format','Position',[basex+170 basey+height 60 20],...
		'Callback','global zpformat; zpformat = ~zpformat;zptool(''refresh'');');
	ha = uicontrol('style','edit','position',[basex+50 basey 50 height-13],...
		'backgroundcolor','black','foregroundcolor','white',...
		'Max',50);
	hb = uicontrol('style','edit','position',[basex basey 50 height-13],...
		'backgroundcolor','black','foregroundcolor','white',...
		'Max',50);
	hz = uicontrol('style','edit','position',[basex+110 basey 100 height],...
		'backgroundcolor','black','foregroundcolor','white',...
		'Max',50);
	hp = uicontrol('style','edit','position',[basex+210 basey 100 height],...
		'backgroundcolor','black','foregroundcolor','white',...
		'Max',50);

% filter type indicators
	base = 440;
	hphase = uicontrol('Position',[5 base+45 100 15],'backgroundcolor',[0 0 0],...
		'Style','text','Callback','');
	hreal = uicontrol('Position',[5 base+30 100 15],'backgroundcolor',[0 0 0],...
		'Style','text','Callback','');
	htype = uicontrol('Position',[5 base+15 100 15],'backgroundcolor',[0 0 0],...
		'Style','text','Callback','','foregroundcolor',[1 1 1]);
	hstable = uicontrol('Position',[5 base 100 15],'backgroundcolor',[0 0 0],...
		'Style','text','Callback','');

% realtime zplane position info
	basex = 150; basey = 480;
	hreloc = uicontrol('style','text','position',[basex basey+75 80 15],...
		'backgroundcolor','black','foregroundcolor','white',...
		'horizontalalignment','right');
	himloc = uicontrol('style','text','position',[basex basey+60 80 15],...
		'backgroundcolor','black','foregroundcolor','white',...
		'horizontalalignment','right');
	hmagloc = uicontrol('style','text','position',[basex basey+45 80 15],...
		'backgroundcolor','black','foregroundcolor','white',...
		'horizontalalignment','right');
	hanglocnorm = uicontrol('style','text','position',[basex basey+30 80 15],...
		'backgroundcolor','black','foregroundcolor','white',...
		'horizontalalignment','right');
	hanglocrad = uicontrol('style','text','position',[basex basey+15 80 15],...
		'backgroundcolor','black','foregroundcolor','white',...
		'horizontalalignment','right');
	hanglocdeg = uicontrol('style','text','position',[basex basey 80 15],...
		'backgroundcolor','black','foregroundcolor','white',...
		'horizontalalignment','right');

	% create 3 sets of axes
	subplot(311);
	set(gca,'Position',[.32 .68 .25 .3]);
	set(gca,'XTick',-3:3);
	set(gca,'YTick',-3:3);
	set(gca,'tag','axtop');
	subplot(312);
	set(gca,'Position',[.23 .3822 .65 .245]);
	set(gca,'tag','axmid');
	subplot(313);
	set(gca,'Position',[.23 .07 0.65 0.245]);
	set(gca,'tag','axbot');
	zptool('refresh');
	set(gcf,'windowbuttonmotionfcn','zptool(''idle'');');
	return;

% if the zeros or poles have been changed, update the undo matrix
elseif strcmp(action,'change')
	if(length(z) < undosize/2)
		undo(1,index) = length(z);
		undo(2:(length(z)+1),index) = z;
	end
	if(length(p) < undosize/2)
		undo((undosize/2+1),index) = length(p);
		undo((undosize/2+2):(length(p)+undosize/2+1),index) = p;
	end
	index = index + 1;
	zptool('refresh');
	return;

% refresh the display because there's been a change to the zeros/poles, or the
% user did something which requires a refresh
elseif strcmp(action,'refresh')
	axes(findobj(gcf,'tag','axtop'));
	zplane(z,p);
	xlabel(''); ylabel('');
	set(gca,'XLim',xlim);
	set(gca,'YLim',ylim);
	[b,a] = zp2tf2(z,p,1);
	% normalize b and a
	if sum(b) > 10^(-9) b = b ./ sum(b); end
	if sum(a) > 10^(-9) a = a ./ sum(a); end
	xlims = get(gca,'XLim');
	ylims = get(gca,'YLim');
	rangey = ylims(2) - ylims(1);

	% update the phase indicator
	MP = 0;
	if isempty(p) & isempty(z) MP = 1;
	elseif isempty(p) if max(abs(z)) < 1 MP = 1; end
	elseif isempty(z) if max(abs(p)) < 1 MP = 1; end
	elseif (max(abs(z)) < 1) & (max(abs(p)) < 1) MP = 1;
	end
	if MP set(hphase,'string','Min Phase','foregroundcolor',[0 1 0]);
	else set(hphase,'string','Non Min Phase','foregroundcolor',[1 1 0]);
	end

	% update the real/complex indicator
	if(max(abs(imag(a))) < tol) & (max(abs(imag(b))) < tol)
		set(hreal,'string','REAL','foregroundColor',[0 1 0]);
		a = real(a); b = real(b);
	else set(hreal,'string','COMPLEX','foregroundColor',[1 0 0]);
	end

	if(max(abs(p)) > tol) set(htype,'string','IIR');
	else set(htype,'string','FIR');
	end

	% update the displays for b, a, z, and p
	[strb,stra,strz,strp] = makestr(b,a,z,p);
	set(hb,'string',strb);
	set(ha,'string',stra);
	set(hz,'string',strz);
	set(hp,'string',strp);

	% if the length of the display is too large to view in the main zptool
	% window, make a new window.  if the new window already exists,
	% just make it active
	if(length(find(strb == sprintf('\n'))) > 15) | ...
		length(find(stra == sprintf('\n'))) > 15
	  h = findobj(0,'Name','Coefficients');
	  if h == []
	    h = figure('Name','Coefficients','NumberTitle','off','position',...
		[650 20 350 720]);
	    basex = 10; basey = 10; height = 700;
	    ha2 = uicontrol('style','edit','position',[basex+50 basey 50 height-13],...
		'backgroundcolor','black','foregroundcolor','white',...
		'Max',50);
	    hb2 = uicontrol('style','edit','position',[basex basey 50 height-13],...
		'backgroundcolor','black','foregroundcolor','white',...
		'Max',50);
	    hz2 = uicontrol('style','edit','position',[basex+110 basey 100 height],...
		'backgroundcolor','black','foregroundcolor','white',...
		'Max',50);
	    hp2 = uicontrol('style','edit','position',[basex+210 basey 100 height],...
		'backgroundcolor','black','foregroundcolor','white',...
		'Max',50);
	  else
	    figure(h);
	  end
	  set(hb2,'string',strb);
	  set(ha2,'string',stra);
	  set(hz2,'string',strz);
	  set(hp2,'string',strp);
	end

	% make the main zptool window active again
	figure(findobj(0,'Name','zptool'));

	% update the stability display
	if(max(abs(p)) > 1)
		set(hstable,'string','UNSTABLE','foregroundColor',[1 0 0]);
	else set(hstable,'string','STABLE','foregroundColor',[0 1 0]);
	end

	% draw the middle plot
	zptool('mid');

	% draw the bottom plot
	zptool('bot');
	return;

% draw the middle plot
elseif strcmp(action,'mid')
	axes(findobj(gcf,'tag','axmid'));
	N = sscanf(get(findobj(0,'tag','Nmid'),'string'),'%d');
	do = mid;
	zptool('drawgraph');
	return;
elseif strcmp(action,'newmid')
	N = sscanf(get(findobj(0,'tag','Nmid'),'string'),'%d');
	do = mid;
	zptool('drawgraph');
	return;

% draw the bottom plot
elseif strcmp(action,'bot')
	axes(findobj(gcf,'tag','axbot'));
	N = sscanf(get(findobj(0,'tag','Nbot'),'string'),'%d');
	do = bot;
	zptool('drawgraph');
	return;
elseif strcmp(action,'newbot')
	N = sscanf(get(findobj(0,'tag','Nbot'),'string'),'%d');
	do = bot;
	zptool('drawgraph');
	return;

% draw a plot
elseif strcmp(action,'drawgraph')
	if do == 1
		[h f] = freqz(b,a,N);
	        plot(f/pi,20*log10(abs(h)));
	        set(gca,'xgrid','on','ygrid','on');
	        xlabel('Normalized frequency (Nyquist == 1)'); ylabel('Mag (dB)');
	end
	if do == 2
		[h f] = freqz(b,a,N);
	        plot(f/pi,abs(h));
	        set(gca,'xgrid','on','ygrid','on');
	        xlabel('Normalized frequency (Nyquist == 1)'); ylabel('Mag');
	end
	if do == 3
		[h f] = freqz(b,a,N);
        	plot(f/pi,angle(h)*180/pi);
	       set(gca,'xgrid','on','ygrid','on');
	       xlabel('Normalized frequency (Nyquist == 1)');
		ylabel('Phase (degrees)');
 	end
	if do == 4
		[h f] = freqz(b,a,N);
        	plot(f/pi,unwrap(angle(h))*180/pi);
	       set(gca,'xgrid','on','ygrid','on');
	       xlabel('Normalized frequency (Nyquist == 1)');
		ylabel('Phase (degrees)');
	end
	if do == 5
		[gd f] = grpdelay(b,a,N);
	       plot(f/pi,gd);
	       xlabel('Normalized frequency (Nyquist == 1)');
		ylabel('Group delay (in samples)');
		set(gca,'xgrid','on','ygrid','on');
	end
	if do == 6
		[h t] = impz(b,a,N);
		stem(t,abs(h));
		set(gca,'xlim',[t(1) t(length(t))]);
		xlabel('Sample number');
		ylabel('h[n]');
	end
	if do == 7
		y = filter(b,a,ones(1,N));
		stem(0:(N-1),abs(y));
		set(gca,'XLim',[0 N-1]);
		xlabel('Sample Number');
		ylabel('y[n]');
	end
	return;

% change the zeros and poles to their previous definition and update the
% index into the undo matrix to reflect the change
elseif strcmp(action,'undo')
	index = index - 2;
	if index < 1 z = []; p = []; index = 2;
	else
	  if(undo(1,index) == 0) z = [];
	  else z = undo(2:(undo(1,index)+1),index);
	  end
	  if(undo(undosize/2+1,index) == 0) p = [];
	  else p = undo((undosize/2+2):(undo(undosize/2+1,index)+undosize/2+1),index);
	  end
	  zptool('change');
	end
	return;

% undo the undo
elseif strcmp(action,'redo')
	if(index<=size(undo,2))
	  if(undo(1,index) == 0) z = [];
	  else z = undo(2:(undo(1,index)+1),index);
	  end
	  if(undo(undosize/2+1,index) == 0) p = [];
	  else p = undo((undosize/2+2):(undo(undosize/2+1,index)+undosize/2+1),index);
	  end
	  zptool('change');
	end
	return;

% if zptool's not doing anything and the cursor is in the zplane window
% display its position
elseif strcmp(action,'idle')
	if strcmp(get(gcf,'Name'),'zptool')
	if ~strcmp(findobj(gco,'style'),'edit')
	axes(findobj(gcf,'tag','axtop'));
	pt = get(gca,'currentpoint');
	x = pt(1,1);
	y = pt(1,2);
	if(x > xlim(1) & x < xlim(2) & y > ylim(1) & y < ylim(2))
		str = sprintf('Re: %7.3f',x);
		set(hreloc,'string',str);
		str = sprintf('Im: %7.3f',y);
		set(himloc,'string',str);
		str = sprintf('Mag: %7.3f',abs(x+j*y));
		set(hmagloc,'string',str);
		temp = angle(x+j*y);
		str = sprintf('%7.3f pi',temp/pi);
		set(hanglocnorm,'string',str);
		str = sprintf('%7.3f rad',temp);
		set(hanglocrad,'string',str);
		str = sprintf('%7.3f deg',temp * 180 / pi);
		set(hanglocdeg,'string',str);
	else
		if get(hreloc,'string') ~= ''
			set(hreloc,'string','');
			set(himloc,'string','');
			set(hmagloc,'string','');
			set(hanglocnorm,'string','');
			set(hanglocrad,'string','');
			set(hanglocdeg,'string','');
		end
	end
	end
	end
	return;

% change the axes for zplane if the user selected zoom in
elseif strcmp(action,'zoomin')
	axes(findobj(gcf,'tag','axtop'));
	xrange = xlim(2) - xlim(1);
	x = xrange/2 + xlim(1);
	yrange = ylim(2) - ylim(1);
	y = yrange/2 + ylim(1);
	xlim = [(x - xrange/3) (x + xrange/3)];
	ylim = [(y - yrange/3) (y + yrange/3)];
	zptool('refresh');
	return;

% change the axes for zplane if the user selected zoom out
elseif strcmp(action,'zoomout')
	axes(findobj(gcf,'tag','axtop'));
	xrange = xlim(2) - xlim(1);
	x = xrange/2 + xlim(1);
	yrange = ylim(2) - ylim(1);
	y = yrange/2 + ylim(1);
	xlim = [(x - xrange/1.5) (x + xrange/1.5)];
	ylim = [(y - yrange/1.5) (y + yrange/1.5)];
	zptool('refresh');
	return;

% change the axes for zplane if the user selected center
elseif strcmp(action,'center')
	axes(findobj(gcf,'tag','axtop'));
	[x y] = ginput(1);
	xlim = [(x - (xlim(2) - xlim(1))/2) (x + (xlim(2) - xlim(1))/2)];
	ylim = [(y - (ylim(2) - ylim(1))/2) (y + (ylim(2) - ylim(1))/2)];
	zptool('refresh');
	return;

% take all the zeros and poles and send them across the unit circle (note the mag
% response will be unchanged)
elseif strcmp(action,'reflectall')
	z = 1./conj(z);
	p = 1./conj(p);
	zptool('change');
	return;

% make all the poles zeros, and make all the zeros poles
elseif strcmp(action,'invert')
	temp = z; z = p; p = temp; zptool('change');
      return;

% add the needed zeros/poles to make the current system allpass
elseif strcmp(action,'AP')
	temp = p;
	p = [p; 1./conj(z)];
	z = [z; 1./conj(temp)];
	zptool('change');
	return;

% if the user wants filtdemo, run it if it is inactive or bring it to the
% foreground if it is already instantiated
elseif strcmp(action,'filtdemo')
	h = findobj(0,'Name','Lowpass Filter Design Demo');
	if(h == []) filtdemo;
	else figure(h);
	end;
	return;

% if the user wants to transfer the filter design from filtdemo to zptool, do so
elseif strcmp(action,'filtdemoupdate')
	h = findobj(0,'Name','Lowpass Filter Design Demo');
	if (h ~= [])
	  figure(findobj(0,'Name','Lowpass Filter Design Demo'));
	  [b a] = filtdemo('getfilt');

	  % avoid the improper notification from tfchk
	  if(length(a) == 1)
	    a = [a zeros(1,length(b)-1)];
	    [z p k] = tf2zp(b,a);
	    p = [];
	    a = 1;
	  else
	    [z p k] = tf2zp(b,a);
	  end
	  figure(findobj(0,'Name','zptool'));
	  zptool('change');
	end
	return;

% if we're exiting, remove filtdemo (if active), the coefficients window
% (if active) and zptool
elseif strcmp(action,'exit')
	h = findobj(0,'Name','Lowpass Filter Design Demo');
	close(h);
	h = findobj(0,'Name','zptool');
	close(h);
	h = findobj(0,'Name','Coefficients');
	close(h);
	return;

% scale all the z and p locations by the value specified by the user
elseif strcmp(action,'scale')
	val = sscanf(get(hscale,'string'),'%f');
	z = z .* val;
	p = p .* val;
	zptool('change');
	return;
end
